__version__ = '2.0.0-a017'

from rltk.record import Record, AutoGeneratedRecord,\
    cached_property, generate_record_property_cache, validate_record, remove_raw_object, set_id
from rltk.dataset import Dataset
from rltk.io import *
from rltk.similarity import *
from rltk.blocking import *
from rltk.tokenizer import *
from rltk.evaluation import *
from rltk.utils import candidate_pairs, get_record_pairs
import rltk.cli
import rltk.remote
